#ifndef COCO_STREAM_ADAPTER_H_
#define COCO_STREAM_ADAPTER_H_

// MIT License
//
// Copyright (c) 2019 Cocotec Limited
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
//
// SPDX-License-Identifier: MIT

#include <ostream>
#include <streambuf>

#include "coco/runtime.h"

namespace coco {
inline namespace v1 {
namespace detail {

class StreamBufferAdapter : public std::basic_streambuf<char> {
 public:
  explicit StreamBufferAdapter(Formatter& formatter) : formatter_(formatter) {}

 protected:
  std::streamsize xsputn(const char_type* s, std::streamsize size) COCO_OVERRIDE {
    formatter_.write_data(s, static_cast<size_t>(size));
    return size;
  }

  int overflow(int ch) COCO_OVERRIDE {
    if (traits_type::not_eof(ch)) {
      formatter_.write_char(traits_type::to_char_type(ch));
    }
    return ch;
  }

 private:
  Formatter& formatter_;
};

}  // namespace detail

template <typename T>
void stream_to_formatter(const T& value, Formatter& formatter) {
  detail::StreamBufferAdapter adapter(formatter);
  std::ostream out(&adapter);
  out << value;
}

template <typename T>
void log_stream_to_formatter(const T& value, Formatter& formatter) {
  formatter.log_begin_external_value();
  stream_to_formatter(value, formatter);
  formatter.log_finish_external_value();
}

}  // namespace v1
}  // namespace coco

#endif
